"""Install for Silva Flexible XML
"""

from Products.Silva.install import add_fss_directory_view
from Products.SilvaFlexibleXML import document

def install(root):
    # create the core views from filesystem
    add_fss_directory_view(root.service_views,
                           'SilvaFlexibleXML', __file__, 'views')
    # also register views
    registerViews(root.service_view_registry)

    # security
    root.manage_permission('Add Silva Flexible XML Documents',
                           ['Author', 'Editor', 'ChiefEditor', 'Manager'])
    root.manage_permission('Add Silva Flexible XSLT Reports',
                           ['Author', 'Editor', 'ChiefEditor', 'Manager'])
    
    #root.manage_permission('Add Silva XML Document Versions',
    #                       ['Author', 'Editor', 'ChiefEditor', 'Manager'])

    mapping = root.service_metadata.getTypeMapping()
    mapping.editMappings('', [
        {'type': 'Silva Flexible XML Document Version',
        'chain': 'silva-content, silva-extra'},
        {'type': 'Silva Flexible XSLT Report',
         'chain': 'silva-content, silva-extra'},
        ])
        
def uninstall(root):
    unregisterViews(root.service_view_registry)
    root.service_views.manage_delObjects(['SilvaFlexibleXML'])
    root.service_containerpolicy.unregister('Silva Flexible XML')
    
def is_installed(root):
    return hasattr(root.service_views, 'SilvaFlexibleXML')

def registerViews(reg):
    """Register core views on registry.
    """
    # edit
    reg.register('edit', 'Silva Flexible XML Document',
                 ['edit', 'VersionedContent', 'XMLDocument'])
    reg.register('edit', 'Silva Flexible XSLT Report',
                 ['edit', 'Content', 'XSLTReport'])
    # public
    reg.register('public', 'Silva Flexible XML Document Version',
                 ['public', 'XMLDocument', 'view'])
    reg.register('preview', 'Silva Flexible XML Document Version',
                 ['public', 'XMLDocument', 'preview'])
    reg.register('public', 'Silva Flexible XSLT Report',
                 ['public', 'XSLTReport', 'view'])
    reg.register('preview', 'Silva Flexible XSLT Report',
                 ['public', 'XSLTReport', 'preview'])
    # add
    reg.register('add', 'Silva Flexible XML Document',
                 ['add', 'XMLDocument'])
    reg.register('add', 'Silva Flexible XSLT Report',
                 ['add', 'XSLTReport'])
    
def unregisterViews(reg):
    for meta_type in ['Silva Flexible XML Document', 'Silva Flexible XSLT Report']:
        reg.unregister('edit', meta_type)
        reg.unregister('public', meta_type)
        reg.unregister('preview', meta_type)
        reg.unregister('add', meta_type)
        
