from zope.interface import Interface

class IXMLDocument(Interface):
    pass

class IXSLTReport(Interface):
    pass

class IXMLSource(Interface):

    def set(xml):
        """Set XML as text.

        Validate, if cannot be validated, raise ValidationError.
        """

    def setTree(tree):
        """Set XML as ElementTree object.

        Validate, if cannot be validated, raise ValidationError.
        """
        
    def get():
        """Get XML.
        """

    def getTree():
        """Get XML as ElementTree object.
        """

class IXHTMLSource(IXMLSource):
    pass

class IXSLTSource(IXMLSource):
    pass

class IFeatureSheetSource(IXMLSource):
    pass

class IValidator(Interface):
    def validate(tree):
        """Validate lxml.etree ElementTree against this validator.

        Returns True if validation passes, False if invalid.
        """

class ITransformer(Interface):
    def transform():
        """Transform XML source with this transformer

        Returns the transformed lxml.etree ElementTree.
        """

    def save(tree):
        """Save tree to string according to transformation rules.

        Returns string.
        """

    def transformToString():
        """Transform XML source to string directly.

        Returns string.
        """

class IHTMLTransformer(ITransformer):
    """Transform something to HTML.
    """
