# Copyright (c) 2005 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass
from OFS import SimpleItem

from StringIO import StringIO
import urllib2
from lxml import etree
import urlparse

from zope.interface import implements

# Silva interfaces
from Products.Silva.interfaces import IContent

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva import mangle
from Products.Silva.Content import Content
from Products.Silva.helpers import add_and_edit

from Products.SilvaFlexibleXML import xmlsource
from Products.SilvaFlexibleXML.interfaces import IXSLTSource, IXSLTReport

class XSLTSource(xmlsource.XMLSource):
    implements(IXSLTSource)
    
class XSLTReport(Content, SimpleItem.SimpleItem):
    """An XSLT report.
    """
    
    security = ClassSecurityInfo()

    __implements__ = IContent

    implements(IXSLTReport)
    
    meta_type = "Silva Flexible XSLT Report"

    def __init__ ( self, id, title ):
        XSLTReport.inheritedAttribute('__init__')(self, id, title)
        self._url = None
        self._xslt_source = XSLTSource()

    # MANIPULATORS
    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_xslt_content')
    def set_xslt_content(self, xslt):
        self._xslt_source.set(xslt)

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_url')
    def set_url(self, url):
        self._url = url
        
    # ACCESSORS

    security.declareProtected(
        SilvaPermissions.ReadSilvaContent, 'get_xslt_content')
    def get_xslt_content(self):
        return self._xslt_source.get()

    security.declareProtected(
        SilvaPermissions.ReadSilvaContent, 'get_url')
    def get_url(self):
        return self._url
    
    security.declareProtected(
        SilvaPermissions.View, 'report')
    def report(self):
        # open url in context of where we are if possible
        url = self._url
        if not url.startswith('http://'):
            start_url = self.absolute_url()
            url = urlparse.urljoin(start_url, url)
        xslt = etree.XSLT(self._xslt_source.getTree())
        f = urllib2.urlopen(url)
        xml = f.read()
        f.close()
        result = xslt.apply(etree.parse(StringIO(xml)))
        return xslt.tostring(result)
        
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'is_published')
    def is_published(self):
        return True

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'is_deletable')
    def is_deletable(self):
        return True

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'can_set_title')
    def can_set_title(self):
        return True

InitializeClass(XSLTReport)

manage_addXSLTReportForm = PageTemplateFile(
    "www/xsltreportAdd", globals(),
    __name__='manage_addXSLTReportForm')

def manage_addXSLTReport(self, id, title, REQUEST=None):
    """Add an XSLT report."""
    id = mangle.Id(self, id)
    if not id.isValid():
        return
    id = str(id)
    report = XSLTReport(id, title)
    self._setObject(id, report)
    report = getattr(self, id)
    report.set_title(title)
    if REQUEST is not None:
        add_and_edit(self, id, REQUEST)
    return ''
