from Products.Silva.silvaxml.xmlexport import theXMLExporter, VersionedContentProducer, SilvaBaseProducer
from Products.Silva.silvaxml.xmlimport import NS_URI
from xml.sax import parseString, make_parser
from xml.sax.handler import ContentHandler, feature_namespaces
from sprout.saxext import hookablehandler

from StringIO import StringIO

XML_NS_URI = 'http://infrae.com/ns/silva_xml'

def initializeXMLExportRegistry():
    """Here the actual content types are registered.
    """
    from Products.SilvaFlexibleXML import document
    exporter = theXMLExporter
    exporter.registerProducer(document.Document,
                              XMLDocumentProducer)
    exporter.registerProducer(document.DocumentVersion,
                              XMLDocumentVersionProducer)

class XMLDocumentProducer(VersionedContentProducer):
    def sax(self):
        self.startElement('xmldocument', {'id': self.context.id})
        self.workflow()
        self.versions()
        self.endElement('xmldocument')

class BodyOnlyHandler(hookablehandler.HookableHandler):
    """Pass along only the XML body, not startDocument/endDocument events.
    """
    def startDocument_override(self):
        pass

    def endDocument_override(self):
        pass
    
class XMLDocumentVersionProducer(SilvaBaseProducer):
    def sax(self):
        self.startElement('content', {'version_id': self.context.id})
        self.metadata()
        self.handler.startPrefixMapping('silva-xml', XML_NS_URI)
        xmlsource_id = self.context.get_selected_xmlsource_id()
        if xmlsource_id is None:
            xmlsource_id = ''
        transformer_id = self.context.get_selected_transformer_id()
        if transformer_id is None:
            transformer_id = ''
        self.handler.startElementNS((XML_NS_URI, 'xmldata'), None,
                                    {(None, 'xmlsource_id'): xmlsource_id,
                                     (None, 'transformer_id'): transformer_id})
        self.handler.startPrefixMapping(None, '')
        self.sax_xml_content(self.context.get_xml_content())
        self.handler.endPrefixMapping(None)
        self.handler.endElementNS((XML_NS_URI, 'xmldata'), None)
        self.handler.endPrefixMapping('silva-xml')
        self.endElement('content')

    def sax_xml_content(self, xml):
        parser = make_parser()
        parser.setFeature(feature_namespaces, 1)
        handler = BodyOnlyHandler(self.handler)
        parser.setContentHandler(handler)
        handler.setDocumentLocator(parser)
        parser.parse(StringIO(xml))
