from sprout.saxext.xmlimport import NULL_SETTINGS
from sprout.saxext.generator import XMLGenerator
from Products.Silva.silvaxml.xmlimport import theXMLImporter, SilvaBaseHandler, NS_URI, generateUniqueId, updateVersionCount, NS_URI
from Products.SilvaFlexibleXML import document
from Products.Silva import mangle
from StringIO import StringIO

XML_NS_URI = 'http://infrae.com/ns/silva_xml'

def initializeXMLImportRegistry():
    """Initialize the global importer object.
    """
    importer = theXMLImporter
    importer.registerHandler((NS_URI, 'xmldocument'), DocumentHandler)

class DocumentHandler(SilvaBaseHandler):
    def getOverrides(self):
        return {
            (NS_URI, 'content'): DocumentContentHandler
            }

    def startElementNS(self, name, qname, attrs):
        if name == (NS_URI, 'xmldocument'):
            id = attrs[(None, 'id')].encode('utf-8')
            uid = generateUniqueId(id, self._parent)
            object = document.Document(uid)
            self.parent()._setObject(uid, object)
            self.setResult(getattr(self._parent, uid))
            
class DocumentContentHandler(SilvaBaseHandler):
    def getOverrides(self):
        return {
            (XML_NS_URI, 'xmldata'): DocumentXMLDataHandler
            }

    def startElementNS(self, name, qname, attrs):
        if name == (NS_URI, 'content'):
            id = attrs[(None, 'version_id')].encode('utf-8')
            if not mangle.Id(self._parent, id).isValid():
                return
            version = document.DocumentVersion(id, '', None)
            self.parent()._setObject(id, version)
            self.setResult(getattr(self._parent, id))
            updateVersionCount(self)
            
    def endElementNS(self, name, qname):
        if name == (NS_URI, 'content'):
            self.result().set_title(
                self._metadata['silva-content']['maintitle'])
            self.storeMetadata()
            self.storeWorkflow()

class DocumentXMLDataHandler(SilvaBaseHandler):

    def startPrefixMapping(self, prefix, url):
        self._generator.startPrefixMapping(prefix, url)

    def endPrefixMapping(self, prefix):
        self._generator.endPrefixMapping(prefix, url)

    def startElementNS(self, name, qname, attrs):
        if name == (XML_NS_URI, 'xmldata'):
            self._f = StringIO()
            self._generator = XMLGenerator(self._f)
            result = self.result()
            xmlsource_id = attrs[(None, 'xmlsource_id')]
            if xmlsource_id == '':
                xmlsource_id = None
            result.set_selected_xmlsource_id(xmlsource_id)
            transformer_id = attrs[(None, 'transformer_id')]
            if transformer_id == '':
                transformer_id = None
            result.set_selected_transformer_id(transformer_id)
            return
        self._generator.startElementNS(name, qname, attrs)
        
    def endElementNS(self, name, qname):
        if name == (XML_NS_URI, 'xmldata'):
            self.result().set_xml_content(self._f.getvalue())
            return
        self._generator.endElementNS(name, qname)

    def characters(self, content):
        self._generator.characters(content)

    def ignorableWhitespace(self, content):
        self._generator.ignoreableWhitespace(content)
        
    def processingInstruction(self, target, data):
        self._generator.processingInstruction(target, data)
