import os, sys

from Testing import ZopeTestCase

ZopeTestCase.installProduct('Five')

import SilvaTestCase
from Products.Silva.silvaxml import xmlexport, xmlimport
from StringIO import StringIO
        
from Products.SilvaFlexibleXML.error import ValidationError

class XMLDocumentTestCase(SilvaTestCase.SilvaTestCase):
    def afterSetUp(self):

        self.root.manage_addProduct['SilvaFlexibleXML'].manage_addDocument(
            'doc', 'Test Doc')
        
    def test_create(self):
        self.assertEquals('doc', self.root.doc.id)
        self.assertEquals('Silva Flexible XML Document', self.root.doc.meta_type)
        self.assertEquals(None, self.root.doc.get_editable().get_xml_content())

    def test_set_xml(self):
        version = self.root.doc.get_editable()
        version.set_xml_content('<a><b/></a>')
        self.assertEquals(
            '<a><b/></a>',
            version.get_xml_content())

    def test_set_invalid_xml(self):
        version = self.root.doc.get_editable()
        self.assertRaises(ValidationError,
                          version.set_xml_content, '<a><c></b></c></a>')
        self.assertEquals(None,
                          version.get_xml_content())

    def test_xml_export(self):
        version = self.root.doc.get_editable()
        version.set_xml_content('<a><b><c>Oh</c></b></a>')
        settings = xmlexport.ExportSettings()
        exporter = xmlexport.theXMLExporter
        exportRoot = xmlexport.SilvaExportRoot(self.root.doc)
        s = exporter.exportToString(exportRoot, settings)
        i = s.find('<silva-xml:xmldata')
        s = s[i:]
        j = s.find('</silva-xml:xmldata>')
        s = s[:j + len('</silva-xml:xmldata>')]
        expected = '<silva-xml:xmldata xmlns="" xmlns:silva-xml="http://infrae.com/ns/silva_xml" transformer_id="" xmlsource_id=""><a><b><c>Oh</c></b></a></silva-xml:xmldata>'
        self.assertEquals(expected, s)

    def test_xml_export_iface_transformer(self):
        version = self.root.doc.get_editable()
        version.set_xml_content('<a><b><c>Oh</c></b></a>')
        version.set_selected_xmlsource_id(
            'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource')
        version.set_selected_transformer_id(
            'Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer')
        settings = xmlexport.ExportSettings()
        exporter = xmlexport.theXMLExporter
        exportRoot = xmlexport.SilvaExportRoot(self.root.doc)
        s = exporter.exportToString(exportRoot, settings)
        i = s.find('<silva-xml:xmldata')
        s = s[i:]
        j = s.find('</silva-xml:xmldata>')
        s = s[:j + len('</silva-xml:xmldata>')]
        expected = '<silva-xml:xmldata xmlns="" xmlns:silva-xml="http://infrae.com/ns/silva_xml" transformer_id="Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer" xmlsource_id="Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource"><a><b><c>Oh</c></b></a></silva-xml:xmldata>'
        self.assertEquals(expected, s)
        
    def test_xml_import(self):
        version = self.root.doc.get_editable()
        version.set_xml_content('<a><b><c>Oh</c></b></a>')
        settings = xmlexport.ExportSettings()
        exporter = xmlexport.theXMLExporter
        exportRoot = xmlexport.SilvaExportRoot(self.root.doc)
        # first create export content
        s = exporter.exportToString(exportRoot, settings)
        # now import it again
        f = StringIO(s)
        xmlimport.importFromFile(f, self.root)
        # and verify whether we got it
        import_doc = self.root.import_of_doc
        self.assertEquals('import_of_doc', import_doc.id)
        editable = import_doc.get_editable()
        self.assertEquals('<a><b><c>Oh</c></b></a>',
                          editable.get_xml_content())
        self.assertEquals(
            None,
            editable.get_selected_xmlsource_id())
        self.assertEquals(None,
                          editable.get_selected_transformer_id())
    
    def test_xml_import_iface_transformer(self):
        version = self.root.doc.get_editable()
        version.set_xml_content('<a><b><c>Oh</c></b></a>')
        version.set_selected_xmlsource_id(
            'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource')
        version.set_selected_transformer_id(
            'Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer')
        settings = xmlexport.ExportSettings()
        exporter = xmlexport.theXMLExporter
        exportRoot = xmlexport.SilvaExportRoot(self.root.doc)
        # first create export content
        s = exporter.exportToString(exportRoot, settings)
        # now import it again
        f = StringIO(s)
        xmlimport.importFromFile(f, self.root)
        # and verify whether we got it
        import_doc = self.root.import_of_doc
        self.assertEquals('import_of_doc', import_doc.id)
        self.assertEquals('<a><b><c>Oh</c></b></a>',
                          import_doc.get_editable().get_xml_content())
        # and whether interfaces and transformer are correct
        self.assertEquals(
            'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource',
            import_doc.get_editable().get_selected_xmlsource_id())
        self.assertEquals(
            'Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer',
            import_doc.get_editable().get_selected_transformer_id())
        
    def test_transformer_ids(self):
        version = self.root.doc.get_editable()
        # normally only transformers for XMLSource are known
        self.assertEquals(
            ['Products.SilvaFlexibleXML.xmlsource.IndentTransformer'],
            version.get_transformer_ids())
        # now we set an interface
        version.set_selected_xmlsource_id(
            'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource')
        
        # now we should have two transformers
        self.assertEquals(
            [
            'Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer',
            'Products.SilvaFlexibleXML.xmlsource.IndentTransformer',
            ],
            version.get_transformer_ids())

        # and we should provide this interface
        self.assertEquals(
            'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource',
            version.get_selected_xmlsource_id())
        
        # remove the interface again
        version.set_selected_xmlsource_id(None)
        
        self.assertEquals(
            ['Products.SilvaFlexibleXML.xmlsource.IndentTransformer'],
            version.get_transformer_ids())
        self.assertEquals(
            None,
            version.get_selected_xmlsource_id())

def test_suite():
    import unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(XMLDocumentTestCase, 'test'))
    return suite

