import os, sys

from Testing import ZopeTestCase
ZopeTestCase.installProduct('Five')

import SilvaTestCase
from StringIO import StringIO

ZopeTestCase.installProduct('SilvaFlexibleXML')

from zope.interface import directlyProvides, implements
from Products.SilvaFlexibleXML import interfaces
from Products.SilvaFlexibleXML.error import ValidationError
from Products.SilvaFlexibleXML import xmlsource
from lxml import etree
from StringIO import StringIO

class XMLSourceTestCase(SilvaTestCase.SilvaTestCase):
        
    def test_xmlsource_interfaces(self):
        expected = [
            (u'Products.SilvaFlexibleXML.interfaces.IFeatureSheetSource',
             interfaces.IFeatureSheetSource),
            (u'Products.SilvaFlexibleXML.interfaces.IXSLTSource',
             interfaces.IXSLTSource),
            (u'Products.SilvaFlexibleXML.interfaces.IXHTMLSource',
             interfaces.IXHTMLSource)]
        expected.sort()

        received = xmlsource.getXMLSourceInterfaces()
        received.sort()
        self.assertEquals(expected, received)

    def test_validator(self):
        data = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Title</title>
</head>
<body>
<p>A paragraph</p>
</body>
</html>
'''
        xhtml = xmlsource.XMLSource(data)
        directlyProvides(xhtml, interfaces.IXHTMLSource)
        self.assert_(interfaces.IValidator(xhtml).validate(xhtml.getTree()))

        data2 = '<a/>'
        f = StringIO(data2)
        self.assert_(not interfaces.IValidator(xhtml).validate(etree.parse(f)))
        
    def test_validation_set(self):
        data = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Title</title>
</head>
<body>
<p>A paragraph</p>
</body>
</html>
'''
        class XHTMLSource(xmlsource.XMLSource):
            implements(interfaces.IXHTMLSource)
            
        self.assertRaises(ValidationError,
                          XHTMLSource, '<a/>')
        
        xhtml = XHTMLSource(data)
        self.assertEquals(data, xhtml.get())

        source = xmlsource.XMLSource()
        directlyProvides(source, interfaces.IXHTMLSource)
        self.assertRaises(ValidationError,
                          source.set, '<a/>')
        self.assertEquals(None, source.get())
        source.set(data)
        self.assertEquals(data, source.get())
        
    def test_transformer(self):
        class FeatureSheetSource(xmlsource.XMLSource):
            implements(interfaces.IFeatureSheetSource)

        data = open(localpath('demo.fst'), 'r').read()
        featuresheet = FeatureSheetSource(data)
        transformers = xmlsource.getTransformers(featuresheet)
        transformers.sort()
        out = xmlsource.getTransformers(featuresheet)[0][1](
            featuresheet).transformToString()
        expected = open(localpath('demo_expected.html'), 'r').read()
        self.assertEquals(expected, out)

    def test_transformer_listing(self):
        class FeatureSheetSource(xmlsource.XMLSource):
            implements(interfaces.IFeatureSheetSource)

        data = open(localpath('demo.fst'), 'r').read()
        featuresheet = FeatureSheetSource(data)
        expected = [
            ('Products.SilvaFlexibleXML.xmlsource.IndentTransformer',
             xmlsource.IndentTransformer),
            ('Products.SilvaFlexibleXML.xmlsource.FeatureSheetTransformer',
             xmlsource.FeatureSheetTransformer),
            ]
        expected.sort()
        received = list(xmlsource.getTransformers(featuresheet))
        received.sort()
        self.assertEquals(expected, received)

    def test_python_paths(self):
        class FeatureSheetSource(xmlsource.XMLSource):
            implements(interfaces.IFeatureSheetSource)

        data = open(localpath('demo.fst'), 'r').read()
        featuresheet = FeatureSheetSource(data)
        for adapter_id, adapter in xmlsource.getTransformers(featuresheet):
            self.assertEquals(adapter_id, xmlsource.getPythonPath(adapter))
            self.assert_(adapter is xmlsource.resolvePythonPath(adapter_id))
            
def localpath(name):
    _testdir = os.path.split(__file__)[0]
    return os.path.join(_testdir, name)

import unittest
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(XMLSourceTestCase, 'test'))
    return suite
